import http.server
import os
import socketserver
import threading
from urllib.parse import urlparse

from albibong.classes.logger import Logger

logger = Logger(__name__, stdout=True, log_to_file=False)
DIRECTORY = os.path.join(os.path.dirname(__file__), "../gui_dist")
INDEXFILE = "/index.html"


class HttpRequestHandler(http.server.SimpleHTTPRequestHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, directory=DIRECTORY)

    def log_message(self, format, *args):
        # logger.info(format % args)
        pass

    def do_GET(self):
        # Parse query data to find out what was requested
        parsedParams = urlparse(self.path)

        # See if the file requested exists
        if os.access(DIRECTORY + parsedParams.path, os.R_OK):
            # File exists, serve it up
            http.server.SimpleHTTPRequestHandler.do_GET(self)
        else:
            # send index.html, but don't redirect
            self.send_response(200)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            with open(DIRECTORY + INDEXFILE, "rb") as fin:
                self.copyfile(fin, self.wfile)


class HttpServerThread(threading.Thread):
    def __init__(self, name, port):
        super().__init__()
        self.name = name
        self.port = port

    def run(self):
        # Create an object of the above class
        handler_object = HttpRequestHandler

        self.http_server = socketserver.TCPServer(("", self.port), handler_object)
        self.http_server.allow_reuse_address = True

        # Star the server
        server_thread = threading.Thread(target=self.http_server.serve_forever)
        server_thread.daemon = True
        server_thread.start()
        logger.info(f"Thread {self.name} started")

    def stop(self):
        self.http_server.server_close()
        logger.info(f"Thread {self.name} stopped")



import json
import logging
import os
from datetime import datetime, timezone

from scapy.all import wrpcap

from albibong.photon_packet_parser import EventData, OperationRequest, OperationResponse

isUsingLogger = False


def use_logger(value):
    global isUsingLogger
    isUsingLogger = value


class Logger(logging.Logger):
    def __init__(
        self,
        name,
        stdout=False,
        log_to_file=True,
        ignore_event=False,
        ignore_request=False,
        ignore_response=False,
    ):
        super().__init__(name)
        self.ignore_event = ignore_event
        self.ignore_request = ignore_request
        self.ignore_response = ignore_response

        self.file_timestamp = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H%M%S")
        log_filepath = os.path.join(
            os.path.dirname(__file__), f"../../../log/log_{self.file_timestamp}.txt"
        )
        os.makedirs(os.path.dirname(log_filepath), exist_ok=True)

        self.setLevel(logging.DEBUG)
        log_formatter = logging.Formatter("%(message)s")

        if log_to_file:
            file_handler = logging.FileHandler(log_filepath)
            file_handler.setFormatter(log_formatter)
            file_handler.setLevel(logging.INFO)
            self.addHandler(file_handler)

        if stdout:
            stdout_handler = logging.StreamHandler()
            stdout_handler.setFormatter(log_formatter)
            stdout_handler.setLevel(logging.INFO)
            self.addHandler(stdout_handler)

        # Load Event and Operation Code for logging
        eventCodeJsonPath = os.path.join(
            os.path.dirname(__file__), "../resources/event_code.json"
        )
        with open(eventCodeJsonPath) as json_file:
            data = json.load(json_file)
            self.event_code = data

        operationCodeJsonPath = os.path.join(
            os.path.dirname(__file__), "../resources/operation_code.json"
        )
        with open(operationCodeJsonPath) as json_file:
            data = json.load(json_file)
            self.operation_code = data

    def get_timestamp(self):
        return datetime.now(timezone.utc).strftime("%H:%M:%S")

    def log_payload(self, payload):
        if not isUsingLogger:
            return

        if type(payload) == EventData:
            self.log_event(payload)
        elif type(payload) == OperationRequest:
            self.log_request(payload)
        elif type(payload) == OperationResponse:
            self.log_response(payload)

    def log_dps_meter_state(self, state: bool):
        text = f"{self.get_timestamp()} DPS METER IS {'RUNNING. The damages below this point will be recorded.' if state else 'PAUSED. The damages below this point will NOT be recorded.'}"
        print(text)
        if isUsingLogger:
            self.info(text)

    def log_event(self, payload: EventData):
        if self.ignore_event:
            return

        # Ignore position update, timesync, and guild updates
        if payload.code == 3:
            return

        parameters = payload.parameters
        if 252 in parameters and parameters[252] in {99, 100, 152}:
            return

        event_code = str(parameters[252]) if 252 in parameters else None
        if event_code in self.event_code:
            event_code = self.event_code[event_code]

        self.info(f"{self.get_timestamp()} event {event_code}")
        self.info(parameters)

    def log_request(self, payload: OperationRequest):
        if self.ignore_request:
            return

        parameters = payload.parameters
        operation_code = str(parameters[253]) if 253 in parameters else None
        if operation_code in self.operation_code:
            operation_code = self.operation_code[operation_code]

        self.info(f"{self.get_timestamp()} request {operation_code}")
        self.info(parameters)

    def log_response(self, payload: OperationResponse):
        if self.ignore_response:
            return

        parameters = payload.parameters
        operation_code = str(parameters[253]) if 253 in parameters else None
        if operation_code in self.operation_code:
            operation_code = self.operation_code[operation_code]

        self.info(f"{self.get_timestamp()} response {operation_code}")
        self.info(parameters)

    def log_error_pcap(self, packet):
        wrpcap(f"../pcap/error_{self.file_timestamp}.pcapng", packet, append=True)



import json
from uuid import UUID

from albibong.classes.coords import Coords
from albibong.classes.item import Item
from albibong.threads.websocket_server import send_event

DIVISOR = 10000


class Character:

    def __init__(
        self,
        id: int,
        uuid: UUID,
        username: str,
        guild: str,
        alliance: str,
        coords: Coords,
        equipment: list[Item] = [Item.get_item_from_code("0")] * 10,
    ):
        self.id = id
        self.uuid = uuid
        self.username = username
        self.guild = guild
        self.alliance = alliance
        self.coords = coords
        self.fame_gained: int = 0
        self.re_spec_gained: int = 0
        self.silver_gained: int = 0
        self.damage_dealt: int = 0
        self.healing_dealt: int = 0
        self.loot: list[str] = []
        self.equipment = equipment

    def handle_health_update(self, parameters):
        if 2 in parameters:
            if parameters[2] < 0:
                if parameters[0] != self.id:
                    self.damage_dealt += abs(parameters[2])
            else:
                self.healing_dealt += abs(parameters[2])

    def update_coords(self, parameters):
        if 3 in parameters:
            self.coords = Coords(parameters[3][0], parameters[3][1])

    def update_fame(self, parameters):
        fame = parameters[2] / DIVISOR
        self.fame_gained += fame
        event = {
            "type": "update_fame",
            "payload": {"username": self.username, "fame_gained": fame},
        }
        send_event(event)

    def update_re_spec(self, parameters):
        re_spec = parameters[2] / DIVISOR
        self.re_spec_gained += re_spec
        event = {
            "type": "update_re_spec",
            "payload": {"username": self.username, "re_spec_gained": re_spec},
        }
        send_event(event)

    def update_loot(self, parameters):
        if 3 in parameters and parameters[3] == True:
            if 5 in parameters:
                silver = parameters[5] / DIVISOR
                self.silver_gained += silver
                event = {
                    "type": "update_silver",
                    "payload": {"username": self.username, "silver_gained": silver},
                }
                send_event(event)
        else:
            self.loot.append(parameters[4])

    def update_equipment(self, equipments):
        new_eq = []
        if equipments != []:
            for eq in equipments:
                obj = Item.get_item_from_code(str(eq))
                new_eq.append(obj)
        self.equipment = new_eq



import functools
import traceback

from scapy.all import UDP, Packet

from albibong.classes.logger import Logger
from albibong.classes.world_data import WorldData, get_world_data
from albibong.photon_packet_parser import (
    EventData,
    OperationRequest,
    OperationResponse,
    PhotonPacketParser,
)

logger = Logger(__name__, ignore_request=True)


def log_payload(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            payload = func(*args, **kwargs)
            logger.log_payload(payload)
        except:
            print(f"ERROR {func.__name__}: {traceback.format_exc()}")

    return wrapper


class PacketHandler:
    def __init__(self):
        self.parser = PhotonPacketParser(
            self.on_event, self.on_request, self.on_response
        )
        self.world_data = get_world_data()

    @log_payload
    def on_event(self, payload: EventData):
        self.world_data.handle_event(payload.parameters)
        return payload

    @log_payload
    def on_request(self, payload: OperationRequest):
        self.world_data.handle_request(payload.parameters)
        return payload

    @log_payload
    def on_response(self, payload: OperationResponse):
        self.world_data.handle_response(payload.parameters)
        return payload

    def handle(self, packet: Packet):
        try:
            payload = packet[UDP].payload.original
            self.parser.handle_payload(payload)
        except Exception as e:
            if type(e) == KeyboardInterrupt:
                raise e

            print(f"ERROR HANDLE: {traceback.format_exc()}")
            logger.log_error_pcap(packet)



class SegmentedPacket:
    def __init__(self, total_length=0, bytes_written=0, total_payload=bytearray(0)):
        self.total_length = total_length
        self.bytes_written = bytes_written
        self.total_payload = total_payload



import io
import struct
import sys
from albibong.photon_packet_parser.protocol16_type import Protocol16Type
from albibong.photon_packet_parser.operation_request import OperationRequest
from albibong.photon_packet_parser.operation_response import OperationResponse
from albibong.photon_packet_parser.event_data import EventData


class Protocol16Deserializer:

    @staticmethod
    def deserialize(input: io.BytesIO, type_code: int):
        if (
            type_code == Protocol16Type.UNKNOWN.value
            or type_code == Protocol16Type.NULL.value
        ):
            return None
        elif type_code == Protocol16Type.DICTIONARY.value:
            return Protocol16Deserializer.deserialize_dictionary(input)
        elif type_code == Protocol16Type.STRINGARRAY.value:
            return Protocol16Deserializer.deserialize_string_array(input)
        elif type_code == Protocol16Type.BYTE.value:
            return Protocol16Deserializer.deserialize_byte(input)
        elif type_code == Protocol16Type.DOUBLE.value:
            return Protocol16Deserializer.deserialize_double(input)
        elif type_code == Protocol16Type.EVENTDATA.value:
            return Protocol16Deserializer.deserialize_event_data(input)
        elif type_code == Protocol16Type.FLOAT.value:
            return Protocol16Deserializer.deserialize_float(input)
        elif type_code == Protocol16Type.INTEGER.value:
            return Protocol16Deserializer.deserialize_integer(input)
        elif type_code == Protocol16Type.HASHTABLE.value:
            return Protocol16Deserializer.deserialize_hash_table(input)
        elif type_code == Protocol16Type.SHORT.value:
            return Protocol16Deserializer.deserialize_short(input)
        elif type_code == Protocol16Type.LONG.value:
            return Protocol16Deserializer.deserialize_long(input)
        elif type_code == Protocol16Type.INTEGERARRAY.value:
            return Protocol16Deserializer.deserialize_integer_array(input)
        elif type_code == Protocol16Type.BOOLEAN.value:
            return Protocol16Deserializer.deserialize_boolean(input)
        elif type_code == Protocol16Type.OPERATIONRESPONSE.value:
            return Protocol16Deserializer.deserialize_operation_response(input)
        elif type_code == Protocol16Type.OPERATIONREQUEST.value:
            return Protocol16Deserializer.deserialize_operation_request(input)
        elif type_code == Protocol16Type.STRING.value:
            return Protocol16Deserializer.deserialize_string(input)
        elif type_code == Protocol16Type.BYTEARRAY.value:
            return Protocol16Deserializer.deserialize_byte_array(input)
        elif type_code == Protocol16Type.ARRAY.value:
            return Protocol16Deserializer.deserialize_array(input)
        elif type_code == Protocol16Type.OBJECTARRAY.value:
            return Protocol16Deserializer.deserialize_object_array(input)
        else:
            raise Exception("Unknown type code: " + str(type_code))

    @staticmethod
    def deserialize_operation_request(input: io.BytesIO):
        code = Protocol16Deserializer.deserialize_byte(input)
        table = Protocol16Deserializer.deserialize_parameter_table(input)
        return OperationRequest(code, table)

    @staticmethod
    def deserialize_operation_response(input: io.BytesIO):
        code = Protocol16Deserializer.deserialize_byte(input)
        return_code = Protocol16Deserializer.deserialize_short(input)
        debug_message = Protocol16Deserializer.deserialize(
            input, Protocol16Deserializer.deserialize_byte(input)
        )
        parameters = Protocol16Deserializer.deserialize_parameter_table(input)
        return OperationResponse(code, return_code, debug_message, parameters)

    @staticmethod
    def deserialize_event_data(input: io.BytesIO):
        code = Protocol16Deserializer.deserialize_byte(input)
        parameters = Protocol16Deserializer.deserialize_parameter_table(input)
        return EventData(code, parameters)

    @staticmethod
    def deserialize_parameter_table(input):
        dictionary_size = Protocol16Deserializer.deserialize_short(input)
        dictionary = {}

        for _ in range(dictionary_size):
            key = Protocol16Deserializer.deserialize_byte(input)
            value_type_code = Protocol16Deserializer.deserialize_byte(input)
            value = Protocol16Deserializer.deserialize(input, value_type_code)
            dictionary[key] = value

        return dictionary

    @staticmethod
    def deserialize_short(input: io.BytesIO):
        buffer = input.read(2)
        return struct.unpack(">h", buffer)[0]

    @staticmethod
    def deserialize_byte(input: io.BytesIO):
        return input.read(1)[0]

    @staticmethod
    def deserialize_boolean(input: io.BytesIO):
        return input.read(1)[0] != 0

    @staticmethod
    def deserialize_integer(input: io.BytesIO):
        buffer = input.read(4)
        return struct.unpack(">i", buffer)[0]

    @staticmethod
    def deserialize_long(input: io.BytesIO):
        buffer = input.read(8)

        if sys.byteorder == "little":
            return struct.unpack(">q", buffer)[0]

        return struct.unpack("<q", buffer)[0]

    @staticmethod
    def deserialize_float(input: io.BytesIO):
        buffer = input.read(4)

        if sys.byteorder == "little":
            return struct.unpack(">f", buffer)[0]

        return struct.unpack("<f", buffer)[0]

    @staticmethod
    def deserialize_double(input: io.BytesIO):
        buffer = input.read(8)

        if sys.byteorder == "little":
            return struct.unpack(">d", buffer)[0]

        return struct.unpack("<d", buffer)[0]

    @staticmethod
    def deserialize_string(input: io.BytesIO):
        string_size = Protocol16Deserializer.deserialize_short(input)

        if string_size == 0:
            return ""

        buffer = input.read(string_size)
        return buffer.decode("utf-8")

    @staticmethod
    def deserialize_byte_array(input: io.BytesIO):
        array_size = Protocol16Deserializer.deserialize_integer(input)

        array = []
        for _ in range(array_size):
            tmp = Protocol16Deserializer.deserialize_byte(input)
            array.append(tmp)

        return array

    @staticmethod
    def deserialize_integer_array(input: io.BytesIO):
        array_size = Protocol16Deserializer.deserialize_integer(input)

        array = []
        for _ in range(array_size):
            tmp = Protocol16Deserializer.deserialize_integer(input)
            array.append(tmp)

        return array

    @staticmethod
    def deserialize_string_array(input: io.BytesIO):
        array_size = Protocol16Deserializer.deserialize_short(input)

        array = []
        for _ in range(array_size):
            tmp = Protocol16Deserializer.deserialize_string(input)
            array.append(tmp)

        return array

    @staticmethod
    def deserialize_object_array(input: io.BytesIO):
        array_size = Protocol16Deserializer.deserialize_short(input)

        array = []
        for _ in range(array_size):
            typeCode = Protocol16Deserializer.deserialize_byte(input)
            tmp = Protocol16Deserializer.deserialize(input, typeCode)
            array.append(tmp)

        return array

    @staticmethod
    def deserialize_array(input: io.BytesIO):
        array_size = Protocol16Deserializer.deserialize_short(input)
        type_code = Protocol16Deserializer.deserialize_byte(input)

        if type_code == Protocol16Type.ARRAY.value:
            result = []
            for _ in range(0, array_size):
                tmp = Protocol16Deserializer.deserialize_array(input)
                result.append(tmp)

            return result
        elif type_code == Protocol16Type.BYTEARRAY.value:
            result = []
            for _ in range(array_size):
                tmp = Protocol16Deserializer.deserialize_byte_array(input)
                result.append(tmp)

            return result
        elif type_code == Protocol16Type.DICTIONARY.value:
            return Protocol16Deserializer.deserialize_dictionary_array(
                input, array_size
            )
        else:
            result = []
            for _ in range(array_size):
                tmp = Protocol16Deserializer.deserialize(input, type_code)
                result.append(tmp)

            return result

    @staticmethod
    def deserialize_dictionary(input: io.BytesIO):
        key_type_code = Protocol16Deserializer.deserialize_byte(input)
        value_type_code = Protocol16Deserializer.deserialize_byte(input)
        dictionary_size = Protocol16Deserializer.deserialize_short(input)
        return Protocol16Deserializer.deserialize_dictionary_elements(
            input, dictionary_size, key_type_code, value_type_code
        )

    @staticmethod
    def deserialize_dictionary_elements(
        input: io.BytesIO,
        dictionary_size: int,
        key_type_code: int,
        value_type_code: int,
    ):
        output = {}
        for _ in range(dictionary_size):
            key = Protocol16Deserializer.deserialize(
                input,
                (
                    Protocol16Deserializer.deserialize_byte(input)
                    if key_type_code == 0 or key_type_code == 42
                    else key_type_code
                ),
            )
            value = Protocol16Deserializer.deserialize(
                input,
                (
                    Protocol16Deserializer.deserialize_byte(input)
                    if value_type_code == 0 or value_type_code == 42
                    else value_type_code
                ),
            )
            output[key] = value

        return output

    @staticmethod
    def deserialize_dictionary_array(input: io.BytesIO, size: int):
        key_type_code = Protocol16Deserializer.deserialize_byte(input)
        value_type_code = Protocol16Deserializer.deserialize_byte(input)
        output = []

        for _ in range(size):
            dictionary_size = Protocol16Deserializer.deserialize_short(input)

            dictionary = {}
            for _ in range(dictionary_size):
                key = None
                if key_type_code > 0:
                    key = Protocol16Deserializer.deserialize(input, key_type_code)
                else:
                    next_key_type_code = Protocol16Deserializer.deserialize_byte(input)
                    key = Protocol16Deserializer.deserialize(input, next_key_type_code)

                value = None
                if value_type_code > 0:
                    value = Protocol16Deserializer.deserialize(input, value_type_code)
                else:
                    next_value_type_code = Protocol16Deserializer.deserialize_byte(
                        input
                    )
                    value = Protocol16Deserializer.deserialize(
                        input, next_value_type_code
                    )

                dictionary[key] = value

            output.append(dictionary)

        return output

    @staticmethod
    def deserialize_hash_table(input: io.BytesIO):
        size = Protocol16Deserializer.deserialize_short(input)
        return Protocol16Deserializer.deserialize_dictionary_elements(
            input, size, Protocol16Type.UNKNOWN, Protocol16Type.UNKNOWN
        )



from enum import Enum


class EventCode(Enum):
    LEAVE = 1
    JOIN_FINISHED = 2
    MOVE = 3
    TELEPORT = 4
    CHANGE_EQUIPMENT = 5
    HEALTH_UPDATE = 6
    ENERGY_UPDATE = 7
    DAMAGE_SHIELD_UPDATE = 8
    CRAFTING_FOCUS_UPDATE = 9
    ACTIVE_SPELL_EFFECTS_UPDATE = 10
    RESET_COOLDOWNS = 11
    ATTACK = 12
    CAST_START = 13
    CHANNELING_UPDATE = 14
    CAST_CANCEL = 15
    CAST_TIME_UPDATE = 16
    CAST_FINISHED = 17
    CAST_SPELL = 18
    CAST_SPELLS = 19
    CAST_HIT = 20
    CAST_HITS = 21
    STORED_TARGETS_UPDATE = 22
    CHANNELING_ENDED = 23
    ATTACK_BUILDING = 24
    INVENTORY_PUT_ITEM = 25
    INVENTORY_DELETE_ITEM = 26
    INVENTORY_STATE = 27
    NEW_CHARACTER = 28
    NEW_EQUIPMENT_ITEM = 29
    NEW_SIEGE_BANNER_ITEM = 30
    NEW_SIMPLE_ITEM = 31
    NEW_FURNITURE_ITEM = 32
    NEW_KILL_TROPHY_ITEM = 33
    NEW_JOURNAL_ITEM = 34
    NEW_LABORER_ITEM = 35
    NEW_EQUIPMENT_ITEM_LEGENDARY_SOUL = 36
    NEW_SIMPLE_HARVESTABLE_OBJECT = 37
    NEW_SIMPLE_HARVESTABLE_OBJECT_LIST = 38
    NEW_HARVESTABLE_OBJECT = 39
    NEW_TREASURE_DESTINATION_OBJECT = 40
    TREASURE_DESTINATION_OBJECT_STATUS = 41
    CLOSE_TREASURE_DESTINATION_OBJECT = 42
    NEW_SILVER_OBJECT = 43
    NEW_BUILDING = 44
    HARVESTABLE_CHANGE_STATE = 45
    MOB_CHANGE_STATE = 46
    FACTION_BUILDING_INFO = 47
    CRAFT_BUILDING_INFO = 48
    REPAIR_BUILDING_INFO = 49
    MELD_BUILDING_INFO = 50
    CONSTRUCTION_SITE_INFO = 51
    PLAYER_BUILDING_INFO = 52
    FARM_BUILDING_INFO = 53
    TUTORIAL_BUILDING_INFO = 54
    LABORER_OBJECT_INFO = 55
    LABORER_OBJECT_JOB_INFO = 56
    MARKET_PLACE_BUILDING_INFO = 57
    HARVEST_START = 58
    HARVEST_CANCEL = 59
    HARVEST_FINISHED = 60
    TAKE_SILVER = 61
    ACTION_ON_BUILDING_START = 62
    ACTION_ON_BUILDING_CANCEL = 63
    ACTION_ON_BUILDING_FINISHED = 64
    ITEM_REROLL_QUALITY_FINISHED = 65
    INSTALL_RESOURCE_START = 66
    INSTALL_RESOURCE_CANCEL = 67
    INSTALL_RESOURCE_FINISHED = 68
    CRAFT_ITEM_FINISHED = 69
    LOGOUT_CANCEL = 70
    CHAT_MESSAGE = 71
    CHAT_SAY = 72
    CHAT_WHISPER = 73
    CHAT_MUTED = 74
    PLAY_EMOTE = 75
    STOP_EMOTE = 76
    SYSTEM_MESSAGE = 77
    UTILITY_TEXT_MESSAGE = 78
    UPDATE_MONEY = 79
    UPDATE_FAME = 80
    UPDATE_LEARNING_POINTS = 81
    UPDATE_RE_SPEC_POINTS = 82
    UPDATE_CURRENCY = 83
    UPDATE_FACTION_STANDING = 84
    UPDATE_MIST_CITY_STANDING = 85
    RESPAWN = 86
    SERVER_DEBUG_LOG = 87
    CHARACTER_EQUIPMENT_CHANGED = 88
    REGENERATION_HEALTH_CHANGED = 89
    REGENERATION_ENERGY_CHANGED = 90
    REGENERATION_MOUNT_HEALTH_CHANGED = 91
    REGENERATION_CRAFTING_CHANGED = 92
    REGENERATION_HEALTH_ENERGY_COMBO_CHANGED = 93
    REGENERATION_PLAYER_COMBO_CHANGED = 94
    DURABILITY_CHANGED = 95
    NEW_LOOT = 96
    ATTACH_ITEM_CONTAINER = 97
    DETACH_ITEM_CONTAINER = 98
    INVALIDATE_ITEM_CONTAINER = 99
    LOCK_ITEM_CONTAINER = 100
    GUILD_UPDATE = 101
    GUILD_PLAYER_UPDATED = 102
    INVITED_TO_GUILD = 103
    GUILD_MEMBER_WORLD_UPDATE = 104
    UPDATE_MATCH_DETAILS = 105
    OBJECT_EVENT = 106
    NEW_MONOLITH_OBJECT = 107
    MONOLITH_HAS_BANNERS_PLACED_UPDATE = 108
    NEW_ORB_OBJECT = 109
    NEW_CASTLE_OBJECT = 110
    NEW_SPELL_EFFECT_AREA = 111
    UPDATE_SPELL_EFFECT_AREA = 112
    NEW_CHAIN_SPELL = 113
    UPDATE_CHAIN_SPELL = 114
    NEW_TREASURE_CHEST = 115
    START_MATCH = 116
    START_ARENA_MATCH_INFOS = 117
    END_ARENA_MATCH = 118
    MATCH_UPDATE = 119
    ACTIVE_MATCH_UPDATE = 120
    NEW_MOB = 121
    DEBUG_AGGRO_INFO = 122
    DEBUG_VARIABLES_INFO = 123
    DEBUG_REPUTATION_INFO = 124
    DEBUG_DIMINISHING_RETURN_INFO = 125
    DEBUG_SMART_CLUSTER_QUEUE_INFO = 126
    CLAIM_ORB_START = 127
    CLAIM_ORB_FINISHED = 128
    CLAIM_ORB_CANCEL = 129
    ORB_UPDATE = 130
    ORB_CLAIMED = 131
    ORB_RESET = 132
    NEW_WAR_CAMP_OBJECT = 133
    NEW_MATCH_LOOT_CHEST_OBJECT = 134
    NEW_ARENA_EXIT = 135
    GUILD_MEMBER_TERRITORY_UPDATE = 136
    INVITED_MERCENARY_TO_MATCH = 137
    CLUSTER_INFO_UPDATE = 138
    FORCED_MOVEMENT = 139
    FORCED_MOVEMENT_CANCEL = 140
    CHARACTER_STATS = 141
    CHARACTER_STATS_KILL_HISTORY = 142
    CHARACTER_STATS_DEATH_HISTORY = 143
    GUILD_STATS = 144
    KILL_HISTORY_DETAILS = 145
    FULL_ACHIEVEMENT_INFO = 146
    FINISHED_ACHIEVEMENT = 147
    ACHIEVEMENT_PROGRESS_INFO = 148
    FULL_ACHIEVEMENT_PROGRESS_INFO = 149
    FULL_TRACKED_ACHIEVEMENT_INFO = 150
    FULL_AUTO_LEARN_ACHIEVEMENT_INFO = 151
    QUEST_GIVER_QUEST_OFFERED = 152
    QUEST_GIVER_DEBUG_INFO = 153
    CONSOLE_EVENT = 154
    TIME_SYNC = 155
    CHANGE_AVATAR = 156
    CHANGE_MOUNT_SKIN = 157
    GAME_EVENT = 158
    KILLED_PLAYER = 159
    DIED = 160
    KNOCKED_DOWN = 161
    UNCONCIOUS = 162
    MATCH_PLAYER_JOINED_EVENT = 163
    MATCH_PLAYER_STATS_EVENT = 164
    MATCH_PLAYER_STATS_COMPLETE_EVENT = 165
    MATCH_TIME_LINE_EVENT_EVENT = 166
    MATCH_PLAYER_MAIN_GEAR_STATS_EVENT = 167
    MATCH_PLAYER_CHANGED_AVATAR_EVENT = 168
    INVITATION_PLAYER_TRADE = 169
    PLAYER_TRADE_START = 170
    PLAYER_TRADE_CANCEL = 171
    PLAYER_TRADE_UPDATE = 172
    PLAYER_TRADE_FINISHED = 173
    PLAYER_TRADE_ACCEPT_CHANGE = 174
    MINI_MAP_PING = 175
    MARKET_PLACE_NOTIFICATION = 176
    DUELLING_CHALLENGE_PLAYER = 177
    NEW_DUELLING_POST = 178
    DUEL_STARTED = 179
    DUEL_ENDED = 180
    DUEL_DENIED = 181
    DUEL_REQUEST_CANCELED = 182
    DUEL_LEFT_AREA = 183
    DUEL_RE_ENTERED_AREA = 184
    NEW_REAL_ESTATE = 185
    MINI_MAP_OWNED_BUILDINGS_POSITIONS = 186
    REAL_ESTATE_LIST_UPDATE = 187
    GUILD_LOGO_UPDATE = 188
    GUILD_LOGO_CHANGED = 189
    PLACEABLE_OBJECT_PLACE = 190
    PLACEABLE_OBJECT_PLACE_CANCEL = 191
    FURNITURE_OBJECT_BUFF_PROVIDER_INFO = 192
    FURNITURE_OBJECT_CHEAT_PROVIDER_INFO = 193
    FARMABLE_OBJECT_INFO = 194
    NEW_UNREAD_MAILS = 195
    MAIL_OPERATION_POSSIBLE = 196
    GUILD_LOGO_OBJECT_UPDATE = 197
    START_LOGOUT = 198
    NEW_CHAT_CHANNELS = 199
    JOINED_CHAT_CHANNEL = 200
    LEFT_CHAT_CHANNEL = 201
    REMOVED_CHAT_CHANNEL = 202
    ACCESS_STATUS = 203
    MOUNTED = 204
    MOUNT_START = 205
    MOUNT_CANCEL = 206
    NEW_TRAVELPOINT = 207
    NEW_ISLAND_ACCESS_POINT = 208
    NEW_EXIT = 209
    UPDATE_HOME = 210
    UPDATE_CHAT_SETTINGS = 211
    RESURRECTION_OFFER = 212
    RESURRECTION_REPLY = 213
    LOOT_EQUIPMENT_CHANGED = 214
    UPDATE_UNLOCKED_GUILD_LOGOS = 215
    UPDATE_UNLOCKED_AVATARS = 216
    UPDATE_UNLOCKED_AVATAR_RINGS = 217
    UPDATE_UNLOCKED_BUILDINGS = 218
    NEW_ISLAND_MANAGEMENT = 219
    NEW_TELEPORT_STONE = 220
    CLOAK = 221
    PARTY_INVITATION = 222
    PARTY_JOIN_REQUEST = 223
    PARTY_JOINED = 224
    PARTY_DISBANDED = 225
    PARTY_PLAYER_JOINED = 226
    PARTY_CHANGED_ORDER = 227
    PARTY_PLAYER_LEFT = 228
    PARTY_LEADER_CHANGED = 229
    PARTY_LOOT_SETTING_CHANGED_PLAYER = 230
    PARTY_SILVER_GAINED = 231
    PARTY_PLAYER_UPDATED = 232
    PARTY_INVITATION_ANSWER = 233
    PARTY_JOIN_REQUEST_ANSWER = 234
    PARTY_MARKED_OBJECTS_UPDATED = 235
    PARTY_ON_CLUSTER_PARTY_JOINED = 236
    PARTY_SET_ROLE_FLAG = 237
    PARTY_INVITE_OR_JOIN_PLAYER_EQUIPMENT_INFO = 238
    SPELL_COOLDOWN_UPDATE = 239
    NEW_HELLGATE_EXIT_PORTAL = 240
    NEW_EXPEDITION_EXIT = 241
    NEW_EXPEDITION_NARRATOR = 242
    EXIT_ENTER_START = 243
    EXIT_ENTER_CANCEL = 244
    EXIT_ENTER_FINISHED = 245
    NEW_QUEST_GIVER_OBJECT = 246
    FULL_QUEST_INFO = 247
    QUEST_PROGRESS_INFO = 248
    QUEST_GIVER_INFO_FOR_PLAYER = 249
    FULL_EXPEDITION_INFO = 250
    EXPEDITION_QUEST_PROGRESS_INFO = 251
    INVITED_TO_EXPEDITION = 252
    EXPEDITION_REGISTRATION_INFO = 253
    ENTERING_EXPEDITION_START = 254
    ENTERING_EXPEDITION_CANCEL = 255
    REWARD_GRANTED = 256
    ARENA_REGISTRATION_INFO = 257
    ENTERING_ARENA_START = 258
    ENTERING_ARENA_CANCEL = 259
    ENTERING_ARENA_LOCK_START = 260
    ENTERING_ARENA_LOCK_CANCEL = 261
    INVITED_TO_ARENA_MATCH = 262
    USING_HELLGATE_SHRINE = 263
    ENTERING_HELLGATE_LOCK_START = 264
    ENTERING_HELLGATE_LOCK_CANCEL = 265
    PLAYER_COUNTS = 266
    IN_COMBAT_STATE_UPDATE = 267
    OTHER_GRABBED_LOOT = 268
    TREASURE_CHEST_USING_START = 269
    TREASURE_CHEST_USING_FINISHED = 270
    TREASURE_CHEST_USING_CANCEL = 271
    TREASURE_CHEST_USING_OPENING_COMPLETE = 272
    TREASURE_CHEST_FORCE_CLOSE_INVENTORY = 273
    LOCAL_TREASURES_UPDATE = 274
    LOOT_CHEST_SPAWNPOINTS_UPDATE = 275
    PREMIUM_CHANGED = 276
    PREMIUM_EXTENDED = 277
    PREMIUM_LIFE_TIME_REWARD_GAINED = 278
    GOLD_PURCHASED = 279
    LABORER_GOT_UPGRADED = 280
    JOURNAL_GOT_FULL = 281
    JOURNAL_FILL_ERROR = 282
    FRIEND_REQUEST = 283
    FRIEND_REQUEST_INFOS = 284
    FRIEND_INFOS = 285
    FRIEND_REQUEST_ANSWERED = 286
    FRIEND_ONLINE_STATUS = 287
    FRIEND_REQUEST_CANCELED = 288
    FRIEND_REMOVED = 289
    FRIEND_UPDATED = 290
    PARTY_LOOT_ITEMS = 291
    PARTY_LOOT_ITEMS_REMOVED = 292
    REPUTATION_UPDATE = 293
    DEFENSE_UNIT_ATTACK_BEGIN = 294
    DEFENSE_UNIT_ATTACK_END = 295
    DEFENSE_UNIT_ATTACK_DAMAGE = 296
    UNRESTRICTED_PVP_ZONE_UPDATE = 297
    REPUTATION_IMPLICATION_UPDATE = 298
    NEW_MOUNT_OBJECT = 299
    MOUNT_HEALTH_UPDATE = 300
    MOUNT_COOLDOWN_UPDATE = 301
    NEW_EXPEDITION_AGENT = 302
    NEW_EXPEDITION_CHECK_POINT = 303
    EXPEDITION_START_EVENT = 304
    VOTE_EVENT = 305
    RATING_EVENT = 306
    NEW_ARENA_AGENT = 307
    BOOST_FARMABLE = 308
    USE_FUNCTION = 309
    NEW_PORTAL_ENTRANCE = 310
    NEW_PORTAL_EXIT = 311
    NEW_RANDOM_DUNGEON_EXIT = 312
    WAITING_QUEUE_UPDATE = 313
    PLAYER_MOVEMENT_RATE_UPDATE = 314
    OBSERVE_START = 315
    MINIMAP_ZERGS = 316
    MINIMAP_SMART_CLUSTER_ZERGS = 317
    PAYMENT_TRANSACTIONS = 318
    PERFORMANCE_STATS_UPDATE = 319
    OVERLOAD_MODE_UPDATE = 320
    DEBUG_DRAW_EVENT = 321
    RECORD_CAMERA_MOVE = 322
    RECORD_START = 323
    CLAIM_POWER_CRYSTAL_START = 324
    CLAIM_POWER_CRYSTAL_CANCEL = 325
    CLAIM_POWER_CRYSTAL_RESET = 326
    CLAIM_POWER_CRYSTAL_FINISHED = 327
    TERRITORY_CLAIM_START = 328
    TERRITORY_CLAIM_CANCEL = 329
    TERRITORY_CLAIM_FINISHED = 330
    TERRITORY_SCHEDULE_RESULT = 331
    TERRITORY_UPGRADE_WITH_POWER_CRYSTAL_RESULT = 332
    RETURNING_POWER_CRYSTAL_START = 333
    RETURNING_POWER_CRYSTAL_FINISHED = 334
    UPDATE_ACCOUNT_STATE = 335
    START_DETERMINISTIC_ROAM = 336
    GUILD_FULL_ACCESS_TAGS_UPDATED = 337
    GUILD_ACCESS_TAG_UPDATED = 338
    GVG_SEASON_UPDATE = 339
    GVG_SEASON_CHEAT_COMMAND = 340
    SEASON_POINTS_BY_KILLING_BOOSTER = 341
    FISHING_START = 342
    FISHING_CAST = 343
    FISHING_CATCH = 344
    FISHING_FINISHED = 345
    FISHING_CANCEL = 346
    NEW_FLOAT_OBJECT = 347
    NEW_FISHING_ZONE_OBJECT = 348
    FISHING_MINI_GAME = 349
    STEAM_ACHIEVEMENT_COMPLETED = 350
    UPDATE_PUPPET = 351
    CHANGE_FLAGGING_FINISHED = 352
    NEW_OUTPOST_OBJECT = 353
    OUTPOST_UPDATE = 354
    OUTPOST_CLAIMED = 355
    OVER_CHARGE_END = 356
    OVER_CHARGE_STATUS = 357
    PARTY_FINDER_FULL_UPDATE = 358
    PARTY_FINDER_UPDATE = 359
    PARTY_FINDER_APPLICANTS_UPDATE = 360
    PARTY_FINDER_EQUIPMENT_SNAPSHOT = 361
    PARTY_FINDER_JOIN_REQUEST_DECLINED = 362
    NEW_UNLOCKED_PERSONAL_SEASON_REWARDS = 363
    PERSONAL_SEASON_POINTS_GAINED = 364
    PERSONAL_SEASON_PAST_SEASON_DATA_EVENT = 365
    EASY_ANTI_CHEAT_MESSAGE_TO_CLIENT = 366
    MATCH_LOOT_CHEST_OPENING_START = 367
    MATCH_LOOT_CHEST_OPENING_FINISHED = 368
    MATCH_LOOT_CHEST_OPENING_CANCEL = 369
    NOTIFY_CRYSTAL_MATCH_REWARD = 370
    CRYSTAL_REALM_FEEDBACK = 371
    NEW_LOCATION_MARKER = 372
    NEW_TUTORIAL_BLOCKER = 373
    NEW_TILE_SWITCH = 374
    NEW_INFORMATION_PROVIDER = 375
    NEW_DYNAMIC_GUILD_LOGO = 376
    NEW_DECORATION = 377
    TUTORIAL_UPDATE = 378
    TRIGGER_HINT_BOX = 379
    RANDOM_DUNGEON_POSITION_INFO = 380
    NEW_LOOT_CHEST = 381
    UPDATE_LOOT_CHEST = 382
    LOOT_CHEST_OPENED = 383
    UPDATE_LOOT_PROTECTED_BY_MOBS_WITH_MINIMAP_DISPLAY = 384
    NEW_SHRINE = 385
    UPDATE_SHRINE = 386
    UPDATE_ROOM = 387
    NEW_MIST_DUNGEON_ROOM_MOB_SOUL = 388
    NEW_HELLGATE_SHRINE = 389
    UPDATE_HELLGATE_SHRINE = 390
    ACTIVATE_HELLGATE_EXIT = 391
    MUTE_PLAYER_UPDATE = 392
    SHOP_TILE_UPDATE = 393
    SHOP_UPDATE = 394
    EASY_ANTI_CHEAT_KICK = 395
    BATTL_EYE_SERVER_MESSAGE = 396
    UNLOCK_VANITY_UNLOCK = 397
    AVATAR_UNLOCKED = 398
    CUSTOMIZATION_CHANGED = 399
    BASE_VAULT_INFO = 400
    GUILD_VAULT_INFO = 401
    BANK_VAULT_INFO = 402
    RECOVERY_VAULT_PLAYER_INFO = 403
    RECOVERY_VAULT_GUILD_INFO = 404
    UPDATE_WARDROBE = 405
    CASTLE_PHASE_CHANGED = 406
    GUILD_ACCOUNT_LOG_EVENT = 407
    NEW_HIDEOUT_OBJECT = 408
    NEW_HIDEOUT_MANAGEMENT = 409
    NEW_HIDEOUT_EXIT = 410
    INIT_HIDEOUT_ATTACK_START = 411
    INIT_HIDEOUT_ATTACK_CANCEL = 412
    INIT_HIDEOUT_ATTACK_FINISHED = 413
    HIDEOUT_MANAGEMENT_UPDATE = 414
    HIDEOUT_UPGRADE_WITH_POWER_CRYSTAL_RESULT = 415
    IP_CHANGED = 416
    SMART_CLUSTER_QUEUE_UPDATE_INFO = 417
    SMART_CLUSTER_QUEUE_ACTIVE_INFO = 418
    SMART_CLUSTER_QUEUE_KICK_WARNING = 419
    SMART_CLUSTER_QUEUE_INVITE = 420
    RECEIVED_GVG_SEASON_POINTS = 421
    TOWER_POWER_POINT_UPDATE = 422
    OPEN_WORLD_ATTACK_SCHEDULE_START = 423
    OPEN_WORLD_ATTACK_SCHEDULE_FINISHED = 424
    OPEN_WORLD_ATTACK_SCHEDULE_CANCEL = 425
    OPEN_WORLD_ATTACK_CONQUER_START = 426
    OPEN_WORLD_ATTACK_CONQUER_FINISHED = 427
    OPEN_WORLD_ATTACK_CONQUER_CANCEL = 428
    OPEN_WORLD_ATTACK_CONQUER_STATUS = 429
    OPEN_WORLD_ATTACK_START = 430
    OPEN_WORLD_ATTACK_END = 431
    NEW_RANDOM_RESOURCE_BLOCKER = 432
    NEW_HOME_OBJECT = 433
    HIDEOUT_OBJECT_UPDATE = 434
    UPDATE_INFAMY = 435
    MINIMAP_POSITION_MARKERS = 436
    NEW_TUNNEL_EXIT = 437
    CORRUPTED_DUNGEON_UPDATE = 438
    CORRUPTED_DUNGEON_STATUS = 439
    CORRUPTED_DUNGEON_INFAMY = 440
    HELLGATE_RESTRICTED_AREA_UPDATE = 441
    HELLGATE_INFAMY = 442
    HELLGATE_STATUS = 443
    HELLGATE_STATUS_UPDATE = 444
    HELLGATE_SUSPENSE = 445
    REPLACE_SPELL_SLOT_WITH_MULTI_SPELL = 446
    NEW_CORRUPTED_SHRINE = 447
    UPDATE_CORRUPTED_SHRINE = 448
    CORRUPTED_SHRINE_USAGE_START = 449
    CORRUPTED_SHRINE_USAGE_CANCEL = 450
    EXIT_USED = 451
    LINKED_TO_OBJECT = 452
    LINK_TO_OBJECT_BROKEN = 453
    ESTIMATED_MARKET_VALUE_UPDATE = 454
    STUCK_CANCEL = 455
    DUNGON_ESCAPE_READY = 456
    FACTION_WARFARE_CLUSTER_STATE = 457
    FACTION_WARFARE_HAS_UNCLAIMED_WEEKLY_REPORTS_EVENT = 458
    SIMPLE_FEEDBACK = 459
    SMART_CLUSTER_QUEUE_SKIP_CLUSTER_ERROR = 460
    XIGN_CODE_EVENT = 461
    BATCH_USE_ITEM_START = 462
    BATCH_USE_ITEM_END = 463
    RED_ZONE_EVENT_CLUSTER_STATUS = 464
    RED_ZONE_PLAYER_NOTIFICATION = 465
    RED_ZONE_WORLD_EVENT = 466
    FACTION_WARFARE_STATS = 467
    UPDATE_FACTION_BALANCE_FACTORS = 468
    FACTION_ENLISTMENT_CHANGED = 469
    UPDATE_FACTION_RANK = 470
    FACTION_WARFARE_CAMPAIGN_REWARDS_UNLOCKED = 471
    FEATURED_FEATURE_UPDATE = 472
    NEW_POWER_CRYSTAL_OBJECT = 473
    MINIMAP_CRYSTAL_POSITION_MARKER = 474
    CARRY_POWER_CRYSTAL_UPDATE = 475
    PICKUP_POWER_CRYSTAL_START = 476
    PICKUP_POWER_CRYSTAL_CANCEL = 477
    PICKUP_POWER_CRYSTAL_FINISHED = 478
    DO_SIMPLE_ACTION_START = 479
    DO_SIMPLE_ACTION_CANCEL = 480
    DO_SIMPLE_ACTION_FINISHED = 481
    NOTIFY_GUEST_ACCOUNT_VERIFIED = 482
    MIGHT_AND_FAVOR_RECEIVED_EVENT = 483
    WEEKLY_PVP_CHALLENGE_REWARD_STATE_UPDATE = 484
    NEW_UNLOCKED_PVP_SEASON_CHALLENGE_REWARDS = 485
    STATIC_DUNGEON_ENTRANCES_DUNGEON_EVENT_STATUS_UPDATES = 486
    STATIC_DUNGEON_DUNGEON_VALUE_UPDATE = 487
    STATIC_DUNGEON_ENTRANCE_DUNGEON_EVENTS_ABORTED = 488
    IN_APP_PURCHASE_CONFIRMED_GOOGLE_PLAY = 489
    FEATURE_SWITCH_INFO = 490
    PARTY_JOIN_REQUEST_ABORTED = 491
    PARTY_INVITE_ABORTED = 492
    PARTY_START_HUNT_REQUEST = 493
    PARTY_START_HUNT_REQUESTED = 494
    PARTY_START_HUNT_REQUEST_ANSWER = 495
    GUILD_INVITE_DECLINED = 496
    CANCEL_MULTI_SPELL_SLOTS = 497
    NEW_VISUAL_EVENT_OBJECT = 498
    CASTLE_CLAIM_PROGRESS = 499
    CASTLE_CLAIM_PROGRESS_LOGO = 500
    TOWN_PORTAL_UPDATE_STATE = 501
    TOWN_PORTAL_FAILED = 502
    CONSUMABLE_VANITY_CHARGES_ADDED = 503
    FESTIVITIES_UPDATE = 504
    NEW_BANNER_OBJECT = 505
    NEW_MISTS_IMMEDIATE_RETURN_EXIT = 506
    MISTS_PLAYER_JOINED_INFO = 507
    NEW_MISTS_STATIC_ENTRANCE = 508
    NEW_MISTS_OPEN_WORLD_EXIT = 509
    NEW_TUNNEL_EXIT_TEMP = 510
    NEW_MISTS_WISP_SPAWN = 511
    MISTS_WISP_SPAWN_STATE_CHANGE = 512
    NEW_MISTS_CITY_ENTRANCE = 513
    NEW_MISTS_CITY_ROADS_ENTRANCE = 514
    MISTS_CITY_ROADS_ENTRANCE_PARTY_STATE_UPDATE = 515
    MISTS_CITY_ROADS_ENTRANCE_CLEAR_STATE_FOR_PARTY = 516
    MISTS_ENTRANCE_DATA_CHANGED = 517
    NEW_MISTS_CAGED_WISP = 518
    MISTS_WISP_CAGE_OPENED = 519
    MISTS_ENTRANCE_PARTY_BINDING_CREATED = 520
    MISTS_ENTRANCE_PARTY_BINDING_CLEARED = 521
    MISTS_ENTRANCE_PARTY_BINDING_INFOS = 522
    NEW_MISTS_BORDER_EXIT = 523
    NEW_MISTS_DUNGEON_EXIT = 524
    LOCAL_QUEST_INFOS = 525
    LOCAL_QUEST_STARTED = 526
    LOCAL_QUEST_ACTIVE = 527
    LOCAL_QUEST_INACTIVE = 528
    LOCAL_QUEST_PROGRESS_UPDATE = 529
    NEW_UNRESTRICTED_PVP_ZONE = 530
    TEMPORARY_FLAGGING_STATUS_UPDATE = 531
    SPELL_TEST_PERFORMANCE_UPDATE = 532
    TRANSFORMATION = 533
    TRANSFORMATION_END = 534
    UPDATE_TRUSTLEVEL = 535
    REVEAL_HIDDEN_TIME_STAMPS = 536
    MODIFY_ITEM_TRAIT_FINISHED = 537
    REROLL_ITEM_TRAIT_VALUE_FINISHED = 538
    HUNT_QUEST_PROGRESS_INFO = 539
    HUNT_STARTED = 540
    HUNT_FINISHED = 541
    HUNT_ABORTED = 542
    HUNT_MISSION_STEP_STATE_UPDATE = 543
    NEW_HUNT_TRACK = 544
    HUNT_MISSION_UPDATE = 545
    HUNT_QUEST_MISSION_PROGRESS_UPDATE = 546
    HUNT_TRACK_USED = 547
    HUNT_TRACK_USEABLE_AGAIN = 548
    MINIMAP_HUNT_TRACK_MARKERS = 549
    NO_TRACKS_FOUND = 550
    HUNT_QUEST_ABORTED = 551
    INTERACT_WITH_TRACK_START = 552
    INTERACT_WITH_TRACK_CANCEL = 553
    INTERACT_WITH_TRACK_FINISHED = 554
    NEW_DYNAMIC_COMPOUND = 555
    LEGENDARY_ITEM_DESTROYED = 556
    ATTUNEMENT_INFO = 557
    TERRITORY_CLAIM_RAIDED_RAW_ENERGY_CRYSTAL_RESULT = 558
    CARRIED_OBJECT_EXPIRY_WARNING = 559
    CARRIED_OBJECT_EXPIRED = 560
    TERRITORY_RAID_START = 561
    TERRITORY_RAID_CANCEL = 562
    TERRITORY_RAID_FINISHED = 563
    TERRITORY_RAID_RESULT = 564
    TERRITORY_MONOLITH_ACTIVE_RAID_STATUS = 565
    TERRITORY_MONOLITH_ACTIVE_RAID_CANCELLED = 566
    MONOLITH_ENERGY_STORAGE_UPDATE = 567
    MONOLITH_NEXT_SCHEDULED_OPEN_WORLD_ATTACK_UPDATE = 568
    MONOLITH_PROTECTED_BUILDINGS_DAMAGE_REDUCTION_UPDATE = 569
    NEW_BUILDING_BASE_EVENT = 570
    NEW_FORTIFICATION_BUILDING = 571
    NEW_CASTLE_GATE_BUILDING = 572
    BUILDING_DURABILITY_UPDATE = 573
    MONOLITH_FORTIFICATION_POINTS_UPDATE = 574
    FORTIFICATION_BUILDING_UPGRADE_INFO = 575
    FORTIFICATION_BUILDINGS_DAMAGE_STATE_UPDATE = 576
    SIEGE_NOTIFICATION_EVENT = 577
    UPDATE_ENEMY_WAR_BANNER_ACTIVE = 578
    TERRITORY_ANNOUNCE_PLAYER_EJECTION = 579
    CASTLE_GATE_SWITCH_USE_STARTED = 580
    CASTLE_GATE_SWITCH_USE_FINISHED = 581
    FORTIFICATION_BUILDING_WILL_DOWNGRADE = 582
    BOT_COMMAND = 583



import queue
import random
import sys
from time import sleep

import webview
from scapy.all import rdpcap

from albibong.classes.logger import Logger
from albibong.classes.packet_handler import PacketHandler
from albibong.threads.http_server import HttpServerThread
from albibong.threads.packet_handler_thread import PacketHandlerThread
from albibong.threads.sniffer_thread import SnifferThread
from albibong.threads.websocket_server import get_ws_server

logger = Logger(__name__, stdout=True, log_to_file=False)
PORT = random.randrange(8500,8999)

def read_pcap(path):
    packet_handler = PacketHandler()
    scapy_cap = rdpcap(path)
    for packet in scapy_cap:
        packet_handler.handle(packet)


def sniff(useWebview):
    _sentinel = object()
    packet_queue = queue.Queue()

    p = SnifferThread(name="sniffer", out_queue=packet_queue, sentinel=_sentinel)

    c = PacketHandlerThread(
        name="packet_handler",
        in_queue=packet_queue,
        sentinel=_sentinel,
    )

    p.start()
    c.start()

    ws_server = get_ws_server()
    ws_server.start()

    if useWebview:
        print("addr",PORT)

        http_server = HttpServerThread(name="http_server", port=PORT)
        http_server.start()

        window = webview.create_window(
            "Albibong",
            url=f"http://localhost:{PORT}/",
            width=1280,
            height=720,
            zoomable=True,
        )

        def on_closing():
            c.stop()
            p.stop()
            ws_server.stop()
            http_server.stop()

        window.events.closing += on_closing

        webview.start()
    else:
        try:
            while True:
                sleep(100)
        except KeyboardInterrupt as e:
            p.stop()
            ws_server.stop()


def main(useWebview=True):
    if len(sys.argv) > 1:
        ws_server = get_ws_server()
        ws_server.start()

        read_pcap(sys.argv[1])
    else:
        sniff(useWebview)



import json
import os
from collections import deque
from uuid import UUID

from albibong.classes.character import Character
from albibong.classes.coords import Coords
from albibong.classes.dungeon import Dungeon
from albibong.classes.item import Item
from albibong.classes.location import Location
from albibong.classes.utils import Utils
from albibong.resources.EventCode import EventCode
from albibong.resources.OperationCode import OperationCode
from albibong.threads.websocket_server import send_event

FILENAME = os.path.join(os.path.expanduser("~"), "Albibong/list_dungeon.json")
os.makedirs(os.path.dirname(FILENAME), exist_ok=True)


class WorldData:

    def __init__(self) -> None:
        self.me: Character = Character(
            id=None,
            uuid=None,
            username="not initialized",
            guild="not initialized",
            alliance="not initialized",
            coords=Coords(0, 0),
        )
        self.current_map: Location = None
        self.current_dungeon: Dungeon = None
        self.characters: dict[str, Character] = {}
        self.char_id_to_username: dict[int, str] = {self.me.id: self.me.username}
        self.char_uuid_to_username: dict[UUID, str] = {self.me.uuid: self.me.username}
        self.change_equipment_log: dict[int, list] = {}
        self.party_members: set[str] = set()
        self.is_dps_meter_running: bool = True

    def handle_response(self, parameters):
        if 253 in parameters:
            if parameters[253] == OperationCode.JOIN.value:
                self.handle_join_response(parameters)
            elif parameters[253] == OperationCode.CHANGE_CLUSTER.value:
                self.set_location(parameters)

    def handle_event(self, parameters):
        if 252 in parameters:
            if parameters[252] == EventCode.OTHER_GRABBED_LOOT.value:
                if 2 in parameters and parameters[2] in self.characters:
                    char: Character = self.characters[parameters[2]]
                    char.update_loot(parameters)
                if self.current_dungeon and parameters[2] == self.me.username:
                    self.current_dungeon.update_loot(parameters)
            elif parameters[252] == EventCode.UPDATE_FAME.value:
                self.me.update_fame(parameters)
                if self.current_dungeon:
                    self.current_dungeon.update_fame(parameters)
            elif parameters[252] == EventCode.UPDATE_RE_SPEC_POINTS.value:
                self.me.update_re_spec(parameters)
                if self.current_dungeon:
                    self.current_dungeon.update_re_spec(parameters)
            elif parameters[252] == EventCode.CHARACTER_EQUIPMENT_CHANGED.value:
                self.change_character_equipment(parameters)
            elif (
                parameters[252] == EventCode.HEALTH_UPDATE.value
                and self.is_dps_meter_running
            ):
                self.update_dps_meter(parameters)
            elif (
                parameters[252] == EventCode.PARTY_JOINED.value
                or parameters[252] == EventCode.PARTY_PLAYER_JOINED.value
                or parameters[252] == EventCode.PARTY_PLAYER_LEFT.value
                or parameters[252] == EventCode.PARTY_DISBANDED.value
            ):
                self.update_party_member(parameters)
            elif parameters[252] == EventCode.NEW_CHARACTER.value:
                self.create_character(
                    id=parameters[0],
                    uuid=parameters[7],
                    username=parameters[1],
                    guild=parameters[8] if 8 in parameters else "",
                    alliance=parameters[49] if 49 in parameters else "",
                    coords=(
                        Coords(parameters[15][0], parameters[15][1])
                        if 15 in parameters
                        else Coords(0, 0)
                    ),
                    equipments=parameters[38] if 38 in parameters else [],
                )
        ...

    def handle_request(self, parameters):
        if 253 in parameters:
            if parameters[253] == OperationCode.MOVE.value and self.me:
                self.me.update_coords(parameters)
        ...

    def end_current_dungeon(self):
        if self.current_dungeon:
            list_dungeon = deque()

            try:
                with open(FILENAME) as json_file:
                    list_dungeon = deque(json.load(json_file))
            except:
                pass

            self.current_dungeon.get_elapsed_time()
            list_dungeon.appendleft(Dungeon.serialize(self.current_dungeon))

            with open(FILENAME, "w") as json_file:
                json.dump(list(list_dungeon), json_file)

            event = {
                "type": "update_dungeon",
                "payload": {"list_dungeon": list(list_dungeon)},
            }
            send_event(event)
            self.current_dungeon = None

    def start_current_dungeon(self, type, name):
        if self.current_dungeon == None:
            new_dungeon = Dungeon(type, name)
            self.current_dungeon = new_dungeon

    def set_dungeon_status(self, check_map, map_type_splitted):
        if "EXPEDITION" in map_type_splitted or "DUNGEON" in map_type_splitted:
            self.start_current_dungeon(type=check_map.type, name=check_map.name)
        elif (
            "EXPEDITION" not in map_type_splitted or "DUNGEON" not in map_type_splitted
        ):
            self.end_current_dungeon()
            return False

    def set_location(self, parameters):
        self.change_equipment_log = {}

        if 1 in parameters:
            check_map = Location.get_location_from_code(parameters[1])
            map_type_splitted = set(check_map.type.split("_"))
            self.set_dungeon_status(check_map, map_type_splitted)

            if "ISLAND" in map_type_splitted or "HIDEOUT" in map_type_splitted:
                check_map.name = f"{parameters[2]}'s {check_map.name}"
                self.current_map = check_map

        elif 0 in parameters:
            check_map = Location.get_location_from_code(parameters[0])
            map_type_splitted = set(check_map.type.split("_"))
            is_dungeon = self.set_dungeon_status(check_map, map_type_splitted)
            if is_dungeon == False:
                self.current_map = Location.get_location_from_code(parameters[0])

        event = {
            "type": "update_location",
            "payload": {
                "map": self.current_map.name if self.current_map else "None",
                "dungeon": (
                    self.current_dungeon.name if self.current_dungeon else "None"
                ),
            },
        }
        send_event(event)

    def update_party_member(self, parameters):
        if parameters[252] == EventCode.PARTY_JOINED.value:
            self.party_members = set(parameters[5])
        elif parameters[252] == EventCode.PARTY_DISBANDED.value:
            self.party_members = {self.me.username}
        elif parameters[252] == EventCode.PARTY_PLAYER_JOINED.value:
            self.party_members.add(parameters[2])
        elif parameters[252] == EventCode.PARTY_PLAYER_LEFT.value:
            uuid = UUID(bytes=bytes(parameters[1]))
            if uuid in self.char_uuid_to_username:
                name = self.char_uuid_to_username[uuid]
                if name == self.me.username:
                    self.party_members = {self.me.username}
        event = {
            "type": "update_dps",
            "payload": {"party_members": self.serialize_party_members()},
        }
        send_event(event)

    def update_dps_meter(self, parameters):
        if 6 in parameters and parameters[6] in self.char_id_to_username:
            username = self.char_id_to_username[parameters[6]]
            if username in self.party_members:
                char: Character = self.characters[username]
                char.handle_health_update(parameters)
                event = {
                    "type": "update_dps",
                    "payload": {"party_members": self.serialize_party_members()},
                }
                send_event(event)

    def change_character_equipment(self, parameters):
        if 2 in parameters:
            if parameters[0] in self.char_id_to_username:
                if self.char_id_to_username[parameters[0]] != "not initialized":
                    char = self.characters[self.char_id_to_username[parameters[0]]]
                    if char.username in self.party_members:
                        char.update_equipment(parameters[2])
                        event = {
                            "type": "update_dps",
                            "payload": {
                                "party_members": self.serialize_party_members()
                            },
                        }
                        send_event(event)
            else:
                self.change_equipment_log[parameters[0]] = parameters[2]

    def handle_join_response(self, parameters):
        # set my character
        self.convert_id_to_name(old_id=self.me.id, new_id=parameters[0], char=self.me)
        self.me.uuid = Utils.convert_int_arr_to_uuid(parameters[1])
        self.me.username = parameters[2]
        self.me.guild = parameters[57] if 57 in parameters else ""
        self.me.alliance = parameters[77] if 77 in parameters else ""
        if self.me.id in self.change_equipment_log:
            self.me.update_equipment(self.change_equipment_log[self.me.id])

        # put self in characters list
        self.characters[self.me.username] = self.me
        self.char_uuid_to_username[self.me.uuid] = self.me.username

        # put self in party
        self.party_members.add(self.me.username)

        # set map my character is currently in
        if parameters[8][0] == "@":
            area = parameters[8].split("@")
            if area[1] == "RANDOMDUNGEON":
                check_map = Location.get_location_from_code(area[1])
                self.start_current_dungeon(type=check_map.type, name=check_map.name)

        event_char = {
            "type": "init_character",
            "payload": {
                "username": self.me.username,
                "fame": self.me.fame_gained,
                "re_spec": self.me.re_spec_gained,
                "silver": self.me.silver_gained,
                "weapon": self.me.equipment[0].image,
            },
        }
        event_map = {
            "type": "update_location",
            "payload": {
                "map": self.current_map.name if self.current_map else "None",
                "dungeon": (
                    Dungeon.serialize(self.current_dungeon)
                    if self.current_dungeon
                    else None
                ),
            },
        }
        event_party = {
            "type": "update_dps",
            "payload": {"party_members": self.serialize_party_members()},
        }
        send_event(event_map)
        send_event(event_char)
        send_event(event_party)

    def convert_id_to_name(self, old_id, new_id, char: Character):
        if old_id in self.char_id_to_username:
            self.char_id_to_username.pop(old_id)  # delete old relative id
        char.id = new_id
        self.char_id_to_username[char.id] = char.username  # add new relative id

    def create_character(
        self,
        id: int,
        uuid: list[int],
        username: str,
        guild: str,
        alliance: str,
        equipments: list[str] = [],
        coords: Coords = Coords(0, 0),
    ):

        # initiate character
        if username not in self.characters:
            char: Character = Character(
                id=id,
                uuid=Utils.convert_int_arr_to_uuid(uuid),
                username=username,
                guild=guild,
                alliance=alliance,
                coords=coords,
            )
            char.update_equipment(equipments)
            self.characters[char.username] = char
            self.char_id_to_username[char.id] = char.username
            self.char_uuid_to_username[char.uuid] = char.username

        # change map
        else:
            char: Character = self.characters[username]
            char.update_equipment(equipments)
            char.coords = coords
            self.convert_id_to_name(old_id=char.id, new_id=id, char=char)

    def serialize_party_members(self):
        serialized = []

        total_damage = 0
        total_heal = 0

        # get total damage and heal for percentage
        for key, value in self.characters.items():
            if key in self.party_members:
                total_damage += value.damage_dealt
                total_heal += value.healing_dealt

        for member in self.party_members:
            # member character initialized
            if member in self.characters:
                char = self.characters[member]
                username = char.username
                damage_dealt = char.damage_dealt
                damage_percent = (
                    round(char.damage_dealt / total_damage * 100, 2)
                    if total_damage > 0
                    else 0
                )
                healing_dealt = char.healing_dealt
                heal_percent = (
                    round(char.healing_dealt / total_heal * 100, 2)
                    if total_heal > 0
                    else 0
                )
                weapon = (
                    Item.serialize(char.equipment[0])["image"]
                    if char.equipment != []
                    else "../public/No Equipment.png"
                )

            # member character not initialized
            else:
                username = member
                damage_dealt = 0
                damage_percent = 0
                healing_dealt = 0
                heal_percent = 0
                weapon = "../public/No Equipment.png"

            data = {
                "username": username,
                "damage_dealt": damage_dealt,
                "damage_percent": damage_percent,
                "healing_dealt": healing_dealt,
                "heal_percent": heal_percent,
                "weapon": weapon,
            }
            serialized.append(data)
        serialized.sort(key=lambda x: x["damage_dealt"], reverse=True)
        return serialized


world_data = WorldData()


def get_world_data():
    return world_data



import json
import os

itemsJsonPath = os.path.join(os.path.dirname(__file__), "../resources/items.json")
with open(itemsJsonPath) as json_file:
    item_data = json.load(json_file)


class Item:

    def __init__(self, id: str, name: str, unique_name: str):
        self.id = id
        self.name = name
        self.unique_name = unique_name
        self.image = (
            "https://render.albiononline.com/v1/item/" + self.unique_name
            if self.unique_name != "None"
            else "../public/No Equipment.png"
        )

    @staticmethod
    def serialize(item):
        return {
            "id": item.id,
            "name": item.name,
            "unique_name": item.unique_name,
            "image": item.image,
        }

    @staticmethod
    def serialize_many(items):
        serialized = []
        for item in items:
            serialized.append(Item.serialize(item))
        return serialized

    @classmethod
    def get_item_from_code(cls, code: str):
        item = item_data[code]

        return cls(id=item["id"], name=item["name"], unique_name=item["unique_name"])



from datetime import datetime
import uuid

DIVISOR = 10000


class Dungeon:

    def __init__(self, type, name) -> None:
        self.id = uuid.uuid4()
        self.type = type
        self.name = name
        self.tier = 0
        self.fame_gained = 0
        self.silver_gained = 0
        self.re_spec_gained = 0
        self.loot = []
        self.start_time = datetime.now()
        self.time_elapsed = 0

    @staticmethod
    def serialize(dungeon):
        return {
            "id": str(dungeon.id),
            "type": dungeon.type,
            "name": dungeon.name,
            "tier": dungeon.tier,
            "fame": dungeon.fame_gained,
            "silver": dungeon.silver_gained,
            "re_spec": dungeon.re_spec_gained,
            "date_time": dungeon.start_time.strftime("%a %d %b %Y, %I:%M%p"),
            "time_elapsed": str(dungeon.time_elapsed).split(".")[0],
        }

    @staticmethod
    def serialize_many(dungeons):
        serialized = []
        for key, value in dungeons.items():
            serialized.append(Dungeon.serialize(value))
        return serialized

    def get_elapsed_time(self):
        end_time = datetime.now()
        self.time_elapsed = end_time - self.start_time

    def update_fame(self, parameters):
        fame = parameters[2] / DIVISOR
        self.fame_gained += fame

    def update_re_spec(self, parameters):
        re_spec = parameters[2] / DIVISOR
        self.re_spec_gained += re_spec

    def update_loot(self, parameters):
        if 3 in parameters and parameters[3] == True:
            silver = parameters[5] / DIVISOR
            self.silver_gained += silver
        else:
            self.loot.append(parameters[4])



import threading
from scapy.all import AsyncSniffer

from albibong.classes.logger import Logger

logger = Logger(__name__, stdout=True, log_to_file=False)


class SnifferThread(threading.Thread):
    def __init__(self, name, out_queue, sentinel):
        super().__init__()
        self.name = name
        self.out_queue = out_queue
        self.sentinel = sentinel
        self.sniffer = AsyncSniffer(filter="udp and port 5056", prn=self.push_packet)

    def push_packet(self, packet):
        self.out_queue.put(packet)

    def run(self):
        logger.info(f"Thread {self.name} started")
        self.sniffer.start()

    def stop(self):
        logger.info(f"Thread {self.name} stopped")
        self.sniffer.stop()
        self.out_queue.put(self.sentinel)



import io
import struct


class NumberSerializer:
    @staticmethod
    def deserialize_int(source: io.BytesIO):
        buffer = source.read(4)
        return struct.unpack(">i", buffer)[0]

    @staticmethod
    def deserialize_short(source: io.BytesIO):
        buffer = source.read(2)
        return struct.unpack(">h", buffer)[0]

    @staticmethod
    def serialize_int(value, target: io.BytesIO):
        target.write(value >> 24)
        target.write(value >> 16)
        target.write(value >> 8)
        target.write(value)
        return target



import json
import os

mapsJsonPath = os.path.join(os.path.dirname(__file__), "../resources/maps.json")
with open(mapsJsonPath) as json_file:
    map_data = json.load(json_file)


class Location:

    def __init__(self, id: str, name: str, type: str):
        self.id = id
        self.name = name
        self.type = type

    def is_black_zone(self):
        if self.type == "OPENPVP_BLACK":
            return True
        elif self.type == "DUNGEON_BLACK":
            return True
        elif self.type == "PASSAGE_BLACK":
            return True
        elif self.type == "TUNNEL":
            return True
        else:
            return False

    def is_red_zone(self):
        if self.type == "OPENPVP_RED":
            return True
        elif self.type == "DUNGEON_RED":
            return True
        elif self.type == "PASSAGE_RED":
            return True
        else:
            return False

    def is_safe_zone(self):
        if self.is_black_zone() == False and self.is_red_zone() == False:
            return True
        else:
            return False

    @classmethod
    def get_location_from_code(cls, code: str):
        location = map_data[code]

        return cls(id=location["id"], name=location["name"], type=location["type"])
